Part II. Configuration of dCache
================================

This part contains descriptions of the components of dCache, their role, functionality within the framework. In short, all information necessary for configuring them.

## Table of Contents

Individual services:

- [The Admin Service](config-admin.md)
    - [First Steps](config-admin.md#first-steps)
    - [Access With SSH](config-admin.md#access-with-ssh)
    - [How to use the Admin Interface](config-admin.md#how-to-use-the-admin-interface)
    - [Creating a new user](config-admin.md#creating-a-new-user)
    - [Direct command execution](config-admin.md#direct-command-execution)
    - [Use of the SSH Admin Interface by scripts](config-admin.md#use-of-the-ssh-admin-interface-by-scripts)

- [The Alarms Service](config-alarms.md)
   - [The Basic Setup](config-alarms.md#the-basic-setup)
   - [Configure where the alarms service is Running](config-alarms.md#configure-where-the-alarms-service-is-running)
   - [Types of Alarms](config-alarms.md#types-of-alarms)
   - [Alarm Priority](config-alarms.md#alarm-priority)
   - [Working with Alarms: Shell Commands](config-alarms.md#working-with-alarms-shell-commands)
   - [Working with Alarms: Admin Commands](config-alarms.md#working-with-alarms-admin-commands)
   - [Advanced Service Configuration: Enabling Automatic Cleanup](config-alarms.md#advanced-service-configuration-enabling-automatic-cleanup)
   - [Advanced Service Configuration: Enabling Email Alerts](config-alarms.md#advanced-service-configuration-enabling-email-alerts)
   - [Miscellaneous Properties of the alarms Service](config-alarms.md#miscellaneous-properties-of-the-alarms-service)
   - [Alarms SPI](config-alarms.md#alarms-spi-service-provider-interface)
   - [Alarms and Kakfa](config-alarms.md#alarms-and-kakfa)

- [The Billing Service](config-billing.md)
   - [The billing log files](config-billing.md#the-billing-log-files)
   - [The billing database](config-billing.md#the-billing-database)
   - [Billing Histogram Data](config-billing.md#billing-histogram-data)
   - [Billing Records](config-billing.md#billing-records)
   - [Upgrading a Previous Installation](config-billing.md#upgrading-a-previous-installation)

- [The Bulk Service](config-bulk.md)
   - [Configuring](config-bulk.md#configuring)
   - [Requests](config-bulk.md#requests)
   - [Activities](config-bulk.md#activities)
   - [Container Design](config-bulk.md#container-design)
   - [Storage and Recovery](config-bulk.md#storage-and-recovery)
   - [Admin Commands](config-bulk.md#admin-commands)

- [The Cleaner Service](config-cleaner.md)
    - [Configuration](config-cleaner.md#configuration)
    - [Disk Cleaner](config-cleaner.md#disk-cleaner)
    - [HSM Cleaner](config-cleaner.md#hsm-cleaner)

- [The Frontend Service](config-frontend.md)
    - [Configuring the Frontend service](config-frontend.md#configuring-the-frontend-service)
    - [Properties controlling monitoring data collection](config-frontend.md#properties-controlling-monitoring-data-collection)
    - [Properties controlling monitoring data access](config-frontend.md#properties-controlling-monitoring-data-access)
    - [Configuring and using the _admin_ role](config-frontend.md#configuring-and-using-the-_admin_-role)

- [The FTP Service](config-ftp.md)
    - [Introduction](config-ftp.md#introduction)
    - [The Control channel](config-ftp.md#the-control-channel)
    - [Data transfers](config-ftp.md#data-transfers)
    - [Configuration examples](config-ftp.md#configuration-examples)

- [dCache History Service](config-history.md)

- [The Info Provider Service](config-info-provider.md)
    - [Internal collection of information](config-info-provider.md#internal-collection-of-information)
    - [Configuring](config-info-provider.md#configuring-the-info-provider)
    - [Testing](config-info-provider.md#testing-the-info-provider)
    - [Publishing dCache information](config-info-provider.md#publishing-dcache-information)
    - [Troubleshooting BDII problems](config-info-provider.md#troubleshooting-bdii-problems)
    - [Updating information](config-info-provider.md#updating-information)

- [The Missing Files Service](config-missing-files.md)
     - [Introduction](config-missing-files.md#introduction)
     - [Setting up](config-missing-files.md#setting-up)

- [dCache as NFSv4.1 Server](config-nfs.md)
     - [Setting up](config-nfs.md#setting-up)
     - [Configuring NFSv4.1 door with GSS-API support](config-nfs.md#configuring-nfsv41-door-with-gss-api-support)
     - [Configuring principal-id mapping for NFS access](config-nfs.md#configuring-principal-id-mapping-for-nfs-access)
     - [Managing group ids](config-nfs.md#managing-group-ids)
     - [Accessing directory tags](config-nfs.md#accessing-directory-tags)

- [The PinManager Service](config-pinmanager.md)
    - [The (Un-)Pinning Concept](config-pinmanager.md#un-pinning-concept)
    - [The pin life cycle](config-pinmanager.md#the-pin-life-cycle)
    - [Configuring](config-pinmanager.md#configuring)

- [The PoolManager Service](config-PoolManager.md)
     - [The Pool Selection Mechanism](config-PoolManager.md#the-pool-selection-mechanism)
     - [Links](config-PoolManager.md#links)
     - [Examples](config-PoolManager.md#examples)
     - [The Partition Manager](config-PoolManager.md#the-partition-manager)
     - [Overview](config-PoolManager.md#overview)
     - [Managing Partitions](config-PoolManager.md#managing-partitions)
     - [Using Partitions](config-PoolManager.md#using-partitions)
     - [Classic Partitions](config-PoolManager.md#classic-partitions)
     - [Link Groups](config-PoolManager.md#link-groups)

- [The Resilience Service](config-resilience.md)
     - [Configuring the Resilience Service](config-resilience.md#configuring-the-resilience-service)
     - [Resilience Home](config-resilience.md#resilience-home)
     - [Admin Commands](config-resilience.md#admin-commands)
     - [Tuning](config-resilience.md#tuning)
     - [Resilience's View of Pool Status](config-resilience.md#resilience-s-view-of-pool-status)
     - [Automatic Staging of Missing CUSTODIAL Replicas](config-resilience.md#automatic-staging-of-missing-custodial-replicas)
     - [Handling of QoS Transitions for Resilient Files](config-resilience.md#handling-of-qos-transitions-for-resilient-files)
     - [Some typical scenarios part - what happens when ...?](config-resilience.md#some-typical-scenarios-part-1-what-happens-when-)
     - [Some typical scenarios part - how do I ...?](config-resilience.md#some-typical-scenarios-part-2-how-do-i-)

- [dCache QoS Service](config-qos-engine.md)
     - [Introduction to the qos service](config-qos-engine.md#introduction-to-the-qos-service)
     - [Configuring the qos service](config-qos-engine.md#configuring-the-qos-service)
     - [QoS Definitions](config-qos-engine.md#qos-definitions)
     - [QoS Home](config-qos-engine.md#qos-home)
     - [Admin Commands](config-qos-engine.md#admin-commands)
     - [Tuning](config-qos-engine.md#tuning)
     - [QoS's View of Pool Status](config-qos-engine.md#qos-s-view-of-pool-status)
     - [Automatic Staging of Missing CUSTODIAL ONLINE replicas](config-qos-engine.md#automatic-staging-of-missing-custodial-online-replicas)
     - [Some typical scenarios part 1 - what happens when ...?](config-qos-engine.md#some-typical-scenarios-part-1-what-happens-when-)
     - [Some typical scenarios part 2 - how do I ...?](config-qos-engine.md#some-typical-scenarios-part-2-how-do-i-)

- [dCache SpaceManager Service](config-spacemanager.md)
     - [Utilization of space reservations for data storage](config-spacemanager.md#utilization-of-space-reservations-for-data-storage)
     - [Properties of space reservation](config-spacemanager.md#properties-of-space-reservation)
     - [Activating Spacemanager](config-spacemanager.md#activating-spacemanager)
     - [Explicit and implicit space reservations for data storage in dCache](config-spacemanager.md#explicit-and-implicit-space-reservations-for-data-storage-in-dcache)
     - [Spacemanager configuration](config-spacemanager.md#spacemanager-configuration)

- [dCache Storage Resource Manager](config-SRM.md)
     - [Configuring the srm service](config-SRM.md#configuring-the-srm-service)
     - [Utilization of Space Reservations for Data Storage](config-SRM.md#utilization-of-space-reservations-for-data-storage)
     - [Configuring the PostgreSQL Database](config-SRM.md#configuring-the-postgresql-database)
     - [General SRM Concepts (for developers)](config-SRM.md#general-srm-concepts-for-developers)

- [dCache SRM Manager Service](config-srmmanager.md)
     - [Configuring the srm manager service](config-srmmanager.md#configuring-the-srm-manager-service)
     - [Tape recall scheduling](config-srmmanager.md#tape-recall-scheduling)

- [The Statistics Service](config-statistics.md)
     - [The Basic Setup](config-statistics.md#the-basic-setup)
     - [The Statistics Web Page](config-statistics.md#the-statistics-web-page)
     - [Explanation of the File Format of the xxx.raw Files](config-statistics.md#explanation-of-the-file-format-of-the-xxxraw-files)

- [dCache as XRoot-Server](config-xrootd.md)
     - [Setting up](config-xrootd.md#setting-up)
     - [Quick tests](config-xrootd.md#quick-tests)
     - [Pool memory requirements](config-xrootd.md#pool-memory-requirements)
     - [Xroot security](config-xrootd.md#xroot-security)
     - [Third-party transfer](config-xrootd.md#xrootd-third-party-transfer)


Overarching concepts:

- [ACLs in dCache](config-acl.md)
   - [Introduction](config-acl.md#introduction)
   - [Configuring ACL support](config-acl.md#configuring-acl-support)
   - [Setting and getting ACLs](config-acl.md#setting-and-getting-acls)
   - [Accessing ACL over NFS mount](config-acl.md#accessing-acl-over-nfs-mount)

- [Chimera](config-chimera.md)
   - [Mounting Chimera through NFS](config-chimera.md#mounting-chimera-through-nfs)
   - [Using dCap with a mounted file system](config-chimera.md#using-dcap-with-a-mounted-file-system)
   - [Communicating with Chimera](config-chimera.md#communicating-with-chimera)
   - [IDs](config-chimera.md#ids)
   - [Directory Tags](config-chimera.md#directory-tags)
   - [Create, List and Read Directory Tags if the Namespace is not Mounted](config-chimera.md#create-list-and-read-directory-tags-if-the-namespace-is-not-mounted)
   - [Create, List and Read Directory Tags if the Namespace is Mounted](config-chimera.md#create-list-and-read-directory-tags-if-the-namespace-is-mounted)
   - [Directory Tags and Command Files](config-chimera.md#directory-tags-and-command-files)
   - [Directory Tags for dCache](config-chimera.md#directory-tags-for-dcache)
   - [Storage Class and Directory Tags](config-chimera.md#storage-class-and-directory-tags)

- [Cell Message Passing](config-message-passing.md)
   - [Naming and addressing](config-message-passing.md#naming-and-addressing)
   - [Routing](config-message-passing.md#routing)
   - [Configuration](config-message-passing.md#configuration)

- [The dCache Tertiary Storage System Interface](config-hsm.md)
   - [Scope of this chapter](config-hsm.md#scope-of-this-chapter)
   - [Requirements for a tertiary storage system](config-hsm.md#requirements-for-a-tertiary-storage-system)
   - [How dCache interacts with a Tertiary Storage System](config-hsm.md#how-dcache-interacts-with-a-tertiary-storage-system)
   - [Details on the TSS-support executable](config-hsm.md#details-on-the-tss-support-executable)
   - [Configuring pools to interact with a Tertiary Storage System](config-hsm.md#configuring-pools-to-interact-with-a-tertiary-storage-system)
   - [How to Store-/Restore files via the Admin Interface](config-hsm.md#how-to-store-restore-files-via-the-admin-interface)
   - [How to monitor what's going on](config-hsm.md#how-to-monitor-whats-going-on)

- [File Hopping](config-hopping.md)
   - [File Hopping on arrival from outside dCache](config-hopping.md#file-hopping-on-arrival-from-outside-dcache)
   - [File mode of replicated files](config-hopping.md#file-mode-of-replicated-files)
   - [File Hopping managed by the PoolManager](config-hopping.md#file-hopping-managed-by-the-poolmanager)
   - [File Hopping managed by the HoppingManager](config-hopping.md#file-hopping-managed-by-the-hoppingmanager)

- [gPlazma: Authorization in dCache](config-gplazma.md)
   - [Basics](config-gplazma.md#basics)
   - [Configuration](config-gplazma.md#configuration)
   - [Plug-ins](config-gplazma.md#plug-ins)
   - [Using X.509 Certificates](config-gplazma.md#using-x509-certificates)
   - [CA Certificates](config-gplazma.md#ca-certificates)
   - [User Certificate](config-gplazma.md#user-certificate)
   - [Host Certificate](config-gplazma.md#host-certificate)
   - [VOMS Proxy Certificate](config-gplazma.md#voms-proxy-certificate)
   - [Configuration files](config-gplazma.md#configuration-files)
   - [storage-authzdb](config-gplazma.md#storage-authzdb)
   - [The gplazmalite-vorole-mapping plug-in](config-gplazma.md#the-gplazmalite-vorole-mapping-plug-in)
   - [Authorizing a VO](config-gplazma.md#authorizing-a-vo)
   - [The kpwd plug-in](config-gplazma.md#the-kpwd-plug-in)
   - [The gridmap plug-in](config-gplazma.md#the-gridmap-plug-in)
   - [gPlazma specific dCache configuration](config-gplazma.md#gplazma-specific-dcache-configuration)
   - [Enabling Username/Password Access for WebDAV](config-gplazma.md#enabling-usernamepassword-access-for-webdav)
   - [Roles](config-gplazma.md#roles)

- [Kafka](kafkaproducer.md)
   - [What do you need to download to enable Kafka in dCache?](kafkaproducer.md#what-do-you-need-to-download-to-enable-kafka-in-dcache)
   - [Apache Kafka Installation.](kafkaproducer.md#apache-kafka-installation)

- [User and Group Quota](quota.md)

- [Stage Protection](config-stage-protection.md)
   - [Configuration of Stage Protection](config-stage-protection.md#configuration-of-stage-protection)
   - [Definition of the White List](config-stage-protection.md#definition-of-the-white-list)

- [Using Space Reservations without SRM](config-write-token.md)
    - [The Space Reservation](config-write-token.md#the-space-reservation)
    - [The WriteToken tag](config-write-token.md#the-writetoken-tag)
    - [Copy a File into the WriteToken](config-write-token.md#copy-a-file-into-the-writetoken)

- [Storage Descriptor / SRR](srr.md)
    - [How the file is generated](srr.md#how-the-file-is-generated)
    - [Setting up dCache for Storage Descriptor](srr.md#setting-up-dcache-for-storage-descriptor)
    - [Configurable properties of dCache Storage Descriptor](srr.md#configurable-properties-of-dcache-storage-descriptor)

- [ZooKeeper](config-zookeeper.md)
    - [Deployment scenarios](config-zookeeper.md#deployment-scenarios)
    - [Configuration](config-zookeeper.md#configuration)
    - [Inspecting ZooKeeper through dCache](config-zookeeper.md#inspecting-zooKeeper-through-dcache)
