/*
 * dCache - http://www.dcache.org/
 *
 * Copyright (C) 2016 Deutsches Elektronen-Synchrotron
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Affero General Public License as
 * published by the Free Software Foundation, either version 3 of the
 * License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Affero General Public License for more details.
 *
 * You should have received a copy of the GNU Affero General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
package diskCacheV111.srm;

import java.rmi.RemoteException;
import org.dcache.srm.v2_2.ISRM;
import org.dcache.srm.v2_2.SrmAbortFilesRequest;
import org.dcache.srm.v2_2.SrmAbortFilesResponse;
import org.dcache.srm.v2_2.SrmAbortRequestRequest;
import org.dcache.srm.v2_2.SrmAbortRequestResponse;
import org.dcache.srm.v2_2.SrmBringOnlineRequest;
import org.dcache.srm.v2_2.SrmBringOnlineResponse;
import org.dcache.srm.v2_2.SrmChangeSpaceForFilesRequest;
import org.dcache.srm.v2_2.SrmChangeSpaceForFilesResponse;
import org.dcache.srm.v2_2.SrmCheckPermissionRequest;
import org.dcache.srm.v2_2.SrmCheckPermissionResponse;
import org.dcache.srm.v2_2.SrmCopyRequest;
import org.dcache.srm.v2_2.SrmCopyResponse;
import org.dcache.srm.v2_2.SrmExtendFileLifeTimeInSpaceRequest;
import org.dcache.srm.v2_2.SrmExtendFileLifeTimeInSpaceResponse;
import org.dcache.srm.v2_2.SrmExtendFileLifeTimeRequest;
import org.dcache.srm.v2_2.SrmExtendFileLifeTimeResponse;
import org.dcache.srm.v2_2.SrmGetPermissionRequest;
import org.dcache.srm.v2_2.SrmGetPermissionResponse;
import org.dcache.srm.v2_2.SrmGetRequestSummaryRequest;
import org.dcache.srm.v2_2.SrmGetRequestSummaryResponse;
import org.dcache.srm.v2_2.SrmGetRequestTokensRequest;
import org.dcache.srm.v2_2.SrmGetRequestTokensResponse;
import org.dcache.srm.v2_2.SrmGetSpaceMetaDataRequest;
import org.dcache.srm.v2_2.SrmGetSpaceMetaDataResponse;
import org.dcache.srm.v2_2.SrmGetSpaceTokensRequest;
import org.dcache.srm.v2_2.SrmGetSpaceTokensResponse;
import org.dcache.srm.v2_2.SrmGetTransferProtocolsRequest;
import org.dcache.srm.v2_2.SrmGetTransferProtocolsResponse;
import org.dcache.srm.v2_2.SrmLsRequest;
import org.dcache.srm.v2_2.SrmLsResponse;
import org.dcache.srm.v2_2.SrmMkdirRequest;
import org.dcache.srm.v2_2.SrmMkdirResponse;
import org.dcache.srm.v2_2.SrmMvRequest;
import org.dcache.srm.v2_2.SrmMvResponse;
import org.dcache.srm.v2_2.SrmPingRequest;
import org.dcache.srm.v2_2.SrmPingResponse;
import org.dcache.srm.v2_2.SrmPrepareToGetRequest;
import org.dcache.srm.v2_2.SrmPrepareToGetResponse;
import org.dcache.srm.v2_2.SrmPrepareToPutRequest;
import org.dcache.srm.v2_2.SrmPrepareToPutResponse;
import org.dcache.srm.v2_2.SrmPurgeFromSpaceRequest;
import org.dcache.srm.v2_2.SrmPurgeFromSpaceResponse;
import org.dcache.srm.v2_2.SrmPutDoneRequest;
import org.dcache.srm.v2_2.SrmPutDoneResponse;
import org.dcache.srm.v2_2.SrmReleaseFilesRequest;
import org.dcache.srm.v2_2.SrmReleaseFilesResponse;
import org.dcache.srm.v2_2.SrmReleaseSpaceRequest;
import org.dcache.srm.v2_2.SrmReleaseSpaceResponse;
import org.dcache.srm.v2_2.SrmReserveSpaceRequest;
import org.dcache.srm.v2_2.SrmReserveSpaceResponse;
import org.dcache.srm.v2_2.SrmResumeRequestRequest;
import org.dcache.srm.v2_2.SrmResumeRequestResponse;
import org.dcache.srm.v2_2.SrmRmRequest;
import org.dcache.srm.v2_2.SrmRmResponse;
import org.dcache.srm.v2_2.SrmRmdirRequest;
import org.dcache.srm.v2_2.SrmRmdirResponse;
import org.dcache.srm.v2_2.SrmSetPermissionRequest;
import org.dcache.srm.v2_2.SrmSetPermissionResponse;
import org.dcache.srm.v2_2.SrmStatusOfBringOnlineRequestRequest;
import org.dcache.srm.v2_2.SrmStatusOfBringOnlineRequestResponse;
import org.dcache.srm.v2_2.SrmStatusOfChangeSpaceForFilesRequestRequest;
import org.dcache.srm.v2_2.SrmStatusOfChangeSpaceForFilesRequestResponse;
import org.dcache.srm.v2_2.SrmStatusOfCopyRequestRequest;
import org.dcache.srm.v2_2.SrmStatusOfCopyRequestResponse;
import org.dcache.srm.v2_2.SrmStatusOfGetRequestRequest;
import org.dcache.srm.v2_2.SrmStatusOfGetRequestResponse;
import org.dcache.srm.v2_2.SrmStatusOfLsRequestRequest;
import org.dcache.srm.v2_2.SrmStatusOfLsRequestResponse;
import org.dcache.srm.v2_2.SrmStatusOfPutRequestRequest;
import org.dcache.srm.v2_2.SrmStatusOfPutRequestResponse;
import org.dcache.srm.v2_2.SrmStatusOfReserveSpaceRequestRequest;
import org.dcache.srm.v2_2.SrmStatusOfReserveSpaceRequestResponse;
import org.dcache.srm.v2_2.SrmStatusOfUpdateSpaceRequestRequest;
import org.dcache.srm.v2_2.SrmStatusOfUpdateSpaceRequestResponse;
import org.dcache.srm.v2_2.SrmSuspendRequestRequest;
import org.dcache.srm.v2_2.SrmSuspendRequestResponse;
import org.dcache.srm.v2_2.SrmUpdateSpaceRequest;
import org.dcache.srm.v2_2.SrmUpdateSpaceResponse;

/**
 * Wraps an SrmHandler to implement the interface Axis calls into for the SRM 2.2 service.
 */
public class SrmHandlerAdapter implements ISRM {

    private final SrmHandler handler;

    public SrmHandlerAdapter(SrmHandler handler) {
        this.handler = handler;
    }

    @Override
    public SrmReserveSpaceResponse srmReserveSpace(
          SrmReserveSpaceRequest srmReserveSpaceRequest)
          throws RemoteException {
        return
              (SrmReserveSpaceResponse)
                    handler.handleRequest("srmReserveSpace", srmReserveSpaceRequest);
    }

    @Override
    public SrmReleaseSpaceResponse srmReleaseSpace(
          SrmReleaseSpaceRequest srmReleaseSpaceRequest)
          throws RemoteException {
        return
              (SrmReleaseSpaceResponse)
                    handler.handleRequest("srmReleaseSpace", srmReleaseSpaceRequest);
    }

    @Override
    public SrmUpdateSpaceResponse srmUpdateSpace(
          SrmUpdateSpaceRequest srmUpdateSpaceRequest)
          throws RemoteException {
        return
              (SrmUpdateSpaceResponse)
                    handler.handleRequest("srmUpdateSpace", srmUpdateSpaceRequest);
    }


    @Override
    public SrmGetSpaceMetaDataResponse srmGetSpaceMetaData(
          SrmGetSpaceMetaDataRequest srmGetSpaceMetaDataRequest)
          throws RemoteException {
        return
              (SrmGetSpaceMetaDataResponse)
                    handler.handleRequest("srmGetSpaceMetaData", srmGetSpaceMetaDataRequest);
    }


    @Override
    public SrmSetPermissionResponse srmSetPermission(
          SrmSetPermissionRequest srmSetPermissionRequest)
          throws RemoteException {
        return
              (SrmSetPermissionResponse)
                    handler.handleRequest("srmSetPermission", srmSetPermissionRequest);
    }


    @Override
    public SrmCheckPermissionResponse srmCheckPermission(
          SrmCheckPermissionRequest srmCheckPermissionRequest)
          throws RemoteException {
        return
              (SrmCheckPermissionResponse)
                    handler.handleRequest("srmCheckPermission", srmCheckPermissionRequest);
    }

    @Override
    public SrmMkdirResponse srmMkdir(SrmMkdirRequest request) throws RemoteException {
        return
              (SrmMkdirResponse)
                    handler.handleRequest("srmMkdir", request);
    }

    @Override
    public SrmRmdirResponse srmRmdir(SrmRmdirRequest request) throws RemoteException {
        return
              (SrmRmdirResponse)
                    handler.handleRequest("srmRmdir", request);
    }

    @Override
    public SrmCopyResponse srmCopy(SrmCopyRequest request) throws RemoteException {
        return
              (SrmCopyResponse)
                    handler.handleRequest("srmCopy", request);
    }

    @Override
    public SrmRmResponse srmRm(SrmRmRequest request) throws RemoteException {
        return
              (SrmRmResponse)
                    handler.handleRequest("srmRm", request);
    }

    @Override
    public SrmLsResponse srmLs(SrmLsRequest srmLsRequest)
          throws RemoteException {
        return (SrmLsResponse) handler.handleRequest("srmLs", srmLsRequest);
    }

    @Override
    public SrmMvResponse srmMv(SrmMvRequest request)
          throws RemoteException {
        return
              (SrmMvResponse)
                    handler.handleRequest("srmMv", request);
    }

    @Override
    public SrmPrepareToGetResponse srmPrepareToGet(
          SrmPrepareToGetRequest srmPrepareToGetRequest)
          throws RemoteException {
        return
              (SrmPrepareToGetResponse)
                    handler.handleRequest("srmPrepareToGet", srmPrepareToGetRequest);
    }

    @Override
    public SrmPrepareToPutResponse srmPrepareToPut(
          SrmPrepareToPutRequest srmPrepareToPutRequest)
          throws RemoteException {
        return
              (SrmPrepareToPutResponse)
                    handler.handleRequest("srmPrepareToPut", srmPrepareToPutRequest);
    }


    @Override
    public SrmReleaseFilesResponse srmReleaseFiles(
          SrmReleaseFilesRequest srmReleaseFilesRequest)
          throws RemoteException {
        return
              (SrmReleaseFilesResponse)
                    handler.handleRequest("srmReleaseFiles", srmReleaseFilesRequest);
    }

    @Override
    public SrmPutDoneResponse srmPutDone(
          SrmPutDoneRequest srmPutDoneRequest)
          throws RemoteException {
        return
              (SrmPutDoneResponse)
                    handler.handleRequest("srmPutDone", srmPutDoneRequest);
    }

    @Override
    public SrmAbortRequestResponse srmAbortRequest(
          SrmAbortRequestRequest srmAbortRequestRequest)
          throws RemoteException {
        return
              (SrmAbortRequestResponse)
                    handler.handleRequest("srmAbortRequest", srmAbortRequestRequest);
    }

    @Override
    public SrmAbortFilesResponse srmAbortFiles(
          SrmAbortFilesRequest srmAbortFilesRequest)
          throws RemoteException {
        return
              (SrmAbortFilesResponse)
                    handler.handleRequest("srmAbortFiles", srmAbortFilesRequest);
    }

    @Override
    public SrmSuspendRequestResponse srmSuspendRequest(
          SrmSuspendRequestRequest srmSuspendRequestRequest)
          throws RemoteException {
        return
              (SrmSuspendRequestResponse)
                    handler.handleRequest("srmSuspendRequest", srmSuspendRequestRequest);
    }

    @Override
    public SrmResumeRequestResponse srmResumeRequest(
          SrmResumeRequestRequest srmResumeRequestRequest)
          throws RemoteException {
        return
              (SrmResumeRequestResponse)
                    handler.handleRequest("srmResumeRequest", srmResumeRequestRequest);
    }

    @Override
    public SrmStatusOfGetRequestResponse srmStatusOfGetRequest(
          SrmStatusOfGetRequestRequest srmStatusOfGetRequestRequest)
          throws RemoteException {
        return
              (SrmStatusOfGetRequestResponse)
                    handler.handleRequest("srmStatusOfGetRequest", srmStatusOfGetRequestRequest);
    }

    @Override
    public SrmStatusOfPutRequestResponse srmStatusOfPutRequest(
          SrmStatusOfPutRequestRequest srmStatusOfPutRequestRequest)
          throws RemoteException {
        return
              (SrmStatusOfPutRequestResponse)
                    handler.handleRequest("srmStatusOfPutRequest", srmStatusOfPutRequestRequest);
    }


    @Override
    public SrmStatusOfCopyRequestResponse srmStatusOfCopyRequest(
          SrmStatusOfCopyRequestRequest request)
          throws RemoteException {
        return
              (SrmStatusOfCopyRequestResponse)
                    handler.handleRequest("srmStatusOfCopyRequest", request);
    }

    @Override
    public SrmGetRequestSummaryResponse srmGetRequestSummary(
          SrmGetRequestSummaryRequest srmGetRequestSummaryRequest)
          throws RemoteException {
        return (SrmGetRequestSummaryResponse)
              handler.handleRequest("srmGetRequestSummary", srmGetRequestSummaryRequest);
    }

    @Override
    public SrmExtendFileLifeTimeResponse srmExtendFileLifeTime(
          SrmExtendFileLifeTimeRequest srmExtendFileLifeTimeRequest)
          throws RemoteException {
        return (SrmExtendFileLifeTimeResponse)
              handler.handleRequest("srmExtendFileLifeTime", srmExtendFileLifeTimeRequest);
    }

    @Override
    public SrmStatusOfBringOnlineRequestResponse srmStatusOfBringOnlineRequest(
          SrmStatusOfBringOnlineRequestRequest srmStatusOfBringOnlineRequestRequest)
          throws RemoteException {
        return (SrmStatusOfBringOnlineRequestResponse)
              handler.handleRequest("srmStatusOfBringOnlineRequest",
                    srmStatusOfBringOnlineRequestRequest);
    }

    @Override
    public SrmBringOnlineResponse srmBringOnline(SrmBringOnlineRequest srmBringOnlineRequest)
          throws RemoteException {
        return (SrmBringOnlineResponse)
              handler.handleRequest("srmBringOnline", srmBringOnlineRequest);
    }

    @Override
    public SrmExtendFileLifeTimeInSpaceResponse srmExtendFileLifeTimeInSpace(
          SrmExtendFileLifeTimeInSpaceRequest srmExtendFileLifeTimeInSpaceRequest)
          throws RemoteException {
        return (SrmExtendFileLifeTimeInSpaceResponse)
              handler.handleRequest("srmExtendFileLifeTimeInSpace",
                    srmExtendFileLifeTimeInSpaceRequest);
    }

    @Override
    public SrmStatusOfUpdateSpaceRequestResponse srmStatusOfUpdateSpaceRequest(
          SrmStatusOfUpdateSpaceRequestRequest srmStatusOfUpdateSpaceRequestRequest)
          throws RemoteException {
        return (SrmStatusOfUpdateSpaceRequestResponse)
              handler.handleRequest("srmStatusOfUpdateSpaceRequest",
                    srmStatusOfUpdateSpaceRequestRequest);
    }

    @Override
    public SrmPurgeFromSpaceResponse srmPurgeFromSpace(
          SrmPurgeFromSpaceRequest srmPurgeFromSpaceRequest) throws RemoteException {
        return (SrmPurgeFromSpaceResponse)
              handler.handleRequest("srmPurgeFromSpace", srmPurgeFromSpaceRequest);
    }

    @Override
    public SrmPingResponse srmPing(SrmPingRequest srmPingRequest) throws RemoteException {
        return (SrmPingResponse)
              handler.handleRequest("srmPing", srmPingRequest);
    }

    @Override
    public SrmGetPermissionResponse srmGetPermission(
          SrmGetPermissionRequest srmGetPermissionRequest) throws RemoteException {
        return (SrmGetPermissionResponse)
              handler.handleRequest("srmGetPermission", srmGetPermissionRequest);
    }

    @Override
    public SrmStatusOfReserveSpaceRequestResponse srmStatusOfReserveSpaceRequest(
          SrmStatusOfReserveSpaceRequestRequest srmStatusOfReserveSpaceRequestRequest)
          throws RemoteException {
        return (SrmStatusOfReserveSpaceRequestResponse)
              handler.handleRequest("srmStatusOfReserveSpaceRequest",
                    srmStatusOfReserveSpaceRequestRequest);
    }

    @Override
    public SrmChangeSpaceForFilesResponse srmChangeSpaceForFiles(
          SrmChangeSpaceForFilesRequest srmChangeSpaceForFilesRequest) throws RemoteException {
        return (SrmChangeSpaceForFilesResponse)
              handler.handleRequest("srmChangeSpaceForFiles", srmChangeSpaceForFilesRequest);
    }

    @Override
    public SrmGetTransferProtocolsResponse srmGetTransferProtocols(
          SrmGetTransferProtocolsRequest srmGetTransferProtocolsRequest) throws RemoteException {
        return (SrmGetTransferProtocolsResponse)
              handler.handleRequest("srmGetTransferProtocols", srmGetTransferProtocolsRequest);
    }

    @Override
    public SrmGetRequestTokensResponse srmGetRequestTokens(
          SrmGetRequestTokensRequest srmGetRequestTokensRequest) throws RemoteException {
        return (SrmGetRequestTokensResponse)
              handler.handleRequest("srmGetRequestTokens", srmGetRequestTokensRequest);
    }

    @Override
    public SrmGetSpaceTokensResponse srmGetSpaceTokens(
          SrmGetSpaceTokensRequest srmGetSpaceTokensRequest) throws RemoteException {
        return (SrmGetSpaceTokensResponse)
              handler.handleRequest("srmGetSpaceTokens", srmGetSpaceTokensRequest);
    }

    @Override
    public SrmStatusOfChangeSpaceForFilesRequestResponse srmStatusOfChangeSpaceForFilesRequest(
          SrmStatusOfChangeSpaceForFilesRequestRequest srmStatusOfChangeSpaceForFilesRequestRequest)
          throws RemoteException {
        return (SrmStatusOfChangeSpaceForFilesRequestResponse)
              handler.handleRequest("srmStatusOfChangeSpaceForFilesRequest",
                    srmStatusOfChangeSpaceForFilesRequestRequest);
    }

    @Override
    public SrmStatusOfLsRequestResponse srmStatusOfLsRequest(
          SrmStatusOfLsRequestRequest srmStatusOfLsRequestRequest) throws RemoteException {
        return (SrmStatusOfLsRequestResponse)
              handler.handleRequest("srmStatusOfLsRequest", srmStatusOfLsRequestRequest);
    }

}
